@@ -15,18 +15,29 @@
 # with this program.  If not, see <http://www.gnu.org/licenses/>.
 ### END LICENSE
 
-# Nautilus extension to launch Nautilus Image Manipulator
-# Some useful resources about Nautilus extensions:
-#     http://live.gnome.org/Nautilus/Development/Extensions
-#     http://svn.gnome.org/viewvc/nautilus-python/trunk/examples/
-#     
-# Place a copy of the extension in
-#     ~/.nautilus/python-extensions/
+# Nautilus extension to launch Nautilus Image Manipulator.
+#
+# Place a copy of this extension in
+#   /usr/share/nautilus-python/extensions
 # or
-#     /usr/lib/nautilus/extensions-2.0/python/
+#   ~/.local/share/nautilus-python/extensions/
+#
+# Some useful resources about Nautilus extensions:
+#   http://projects.gnome.org/nautilus-python/
+#   http://git.gnome.org/browse/nautilus-python/tree/examples
+#   http://git.gnome.org/browse/nautilus-python/tree/examples?h=nautilus-3.0
+#
+# Note that as of nautilus-python 0.7.0 the path from where the extensions
+# are loaded has been changed:
+#   http://git.gnome.org/browse/nautilus-python/commit/?id=c97253104e7d6b88803cbef529bd9e298fa8d9e3
+#   http://git.gnome.org/browse/nautilus-python/commit/?h=nautilus-3.0&id=73d2739c4a91d80710b14ff5f192875a34f10a94
 
 
-import os, subprocess, urllib, gettext
+import os
+import subprocess
+import urllib
+
+import gettext
 from gettext import gettext as _
 from gettext import ngettext
 gettext.textdomain('nautilus-image-manipulator')
@@ -43,7 +54,7 @@ class BackgroundImageExtension(GObject.GObject, Nautilus.MenuProvider):
         for ff in images:
             # Remove "file://" and unquote the filename
             args.extend(("-f", urllib.unquote(ff.get_uri()[7:])))
-        retVal = subprocess.call(args)
+        subprocess.Popen(args)
 
     def get_file_items(self, window, files):
         images = []
@@ -56,12 +67,12 @@ class BackgroundImageExtension(GObject.GObject, Nautilus.MenuProvider):
         # image in the selection
         if not images:
             return
-        
+
         extLabel = ngettext("_Resize image", "_Resize images", len(images))
         extTip = ngettext("Resize the selected image",
                           "Resize each selected image",
                           len(images))
-        
+
         item = Nautilus.MenuItem(name='NautilusImageManipulator::resize',
                                  label=extLabel,
                                  tip=extTip)
